// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 * RMF.cc
 *
 *  Created on: Mar 11, 2013
 *      Author: aibarra
 */


#include "RMF.h"
#include "CalServer.h"
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"
#include "param.h"
#include "slatec.h"

RMF::RMF(std::string &fileName) :
	_fileName(fileName)
{
	_rsp = new Response();
	_rsp = this->fixrmf(fileName);
}

Response* RMF::fixrmf(string &rmfname) const
{
   // Response resp;

   /* Open the RMF file */
    DataSet * rmfset =  dataSetServer->open(rmfname, DataSet::Read);

   /* Access RMF matrix */
    Table * resptable=0;

    if (rmfset->hasBlock("SPECRESP"))
       resptable = rmfset->table("SPECRESP");
    else if (rmfset->hasBlock("MATRIX"))
       resptable = rmfset->table("MATRIX");
    else if (rmfset->hasBlock("SPECRESP MATRIX"))
       resptable = rmfset->table("SPECRESP MATRIX");
    else
       errstr<<"MATRIX block not found in rmf file"
             <<Sas::error("NoMatrixBlock");

   /* Read the number of detector channels and Minimum channel */
    int nchan = resptable->attribute("DETCHANS")->asInt();
    //int tlmin4 = resptable->attribute("TLMIN4")->asInt();
    int tlmin4 = 0; // Last line doesn't work - dont know why


   /* Read the matrix arrays */
    ColumnData *cdElo = resptable->column("ENERG_LO")->data(),
               *cdEhi = resptable->column("ENERG_HI")->data(),
               *cdNgp = resptable->column("N_GRP")->data(),
               *cdFch = resptable->column("F_CHAN")->data(),
               *cdNch = resptable->column("N_CHAN")->data();

    Column     *cMat = resptable->column("MATRIX");

    real32 *elo = cdElo->real32Data(),
           *ehi = cdEhi->real32Data();
           //*mat = cdMat->real32Data();

    int16  *n_grp = cdNgp->int16Data(),
           *f_chan = cdFch->int16Data(),
           *n_chan = cdNch->int16Data();

   /* Read the ebounds arrays */
    Table * ebndtable = rmfset->table("EBOUNDS");
    ColumnData *cdEblo = ebndtable->column("E_MIN")->data(),
               *cdEbhi = ebndtable->column("E_MAX")->data();

    real32 *eblo = cdEblo->real32Data(),
           *ebhi = cdEbhi->real32Data();

   /* Manipulate the matrix */
    int nbin = resptable->rows();   // Number of energies
    //int npi = cdEbhi->elements();   // Number of channels
    int npi = ebndtable->rows();   // Number of channels

    //int ngrpmax = cdNgp->elements() / nbin;
    int ngrpmax = cdFch->elements();

   /* Find size of matrix - which is nbin * max(chans) */
    //int maxrmfchans = cdMat->elements() / nbin;

///////f0=sxpar(ha,'TLMIN4',cou=np)+(max(a.f_chan+a.n_chan) gt nchan)

   /* Need to convert the integer stream into a 2-d array */

    int addf0 = 0;
    int f0 = tlmin4;
    int maxfchan=0, minnchan=99999;
    int el1d=0; // Element in 2-d array

    for (int i=0; i<nbin; ++i) {
      //for (int j=0; j<n_grp[i]; ++i) { /// !!!!!!!!!!!!!!!!!
      for (int j=0; j<n_grp[i]; ++j) {    //// !!!!!!!!!!!!!!!
        el1d = ngrpmax * (i) + j;
        //if ((f_chan[i][j] + n_chan[i][j]) > nchan) addf0=1;
        if ((f_chan[el1d] + n_chan[el1d]) > nchan) addf0=1;

        if (f_chan[el1d] > maxfchan) maxfchan=f_chan[el1d];
        if (n_chan[el1d] < minnchan) minnchan=n_chan[el1d];
      }
    }
    f0 = f0 + addf0;

   // Uncompress the matrix
    //if ( maxfchan == 1 && minnchan == nchan ) {
       //outmat=mat;   // Probbaly never happens
   // } else {

      for (int i = 0; i < nbin; ++i) {  // Loop over energies

         // Read the Matrix array (cell) for this energy
          CellData *matrow = cMat->cellData(i);

          vector<double> outmat(npi); // Matrix for one energy (=0)

          int k=0; // Counter for matrix element position
          for (int j=0; j<n_grp[i]; ++j) { // Loop over matrix groups

              el1d = ngrpmax * i + j; // Position in group array

              for (int ind2=0; ind2<n_chan[el1d]; ++ind2) {
                  int unpacked_index = f_chan[el1d]-f0 +ind2;

               // cout<<"IND: "<<unpacked_index<<" "<<npi<<endl;
                //cout<<"IND: "<<f0<<" "<<f_chan[el1d]<<" "<<ind2<<" "<<el1d<<endl;
                  outmat[unpacked_index] = matrow->real32Data()[k+ind2];

                  //int packed_index = i*maxrmfchans + k + ind2;
                  //temp[unpacked_index] = mat[packed_index];
                  //temp[index] = mat[i][k+ind2];
              }
              k=k+n_chan[el1d];
          } // End of loop over matrix elements for this energy

          _rsp->matrix.push_back(outmat);

          delete matrow; matrow=0;   // Remove to avoid glibc error

      } // End of loop over energies
    // }

   // Fill the output structure
    for (int i=0; i<nbin; ++i) {
        _rsp->elo.push_back(elo[i]);
        _rsp->ehi.push_back(ehi[i]);
    }

    for (int i=0; i<npi; ++i) {
        _rsp->eblo.push_back(eblo[i]);
        _rsp->ebhi.push_back(ebhi[i]);
    }

   // Clean
    delete cdElo;
    delete cdEhi;
    delete cdEblo;
    delete cdEbhi;
    delete cdNgp;
    delete cdFch;
    delete cdNch;
    dataSetServer->close(rmfset);
   //

    return _rsp;
}


/*
* writeAttributes: Copies header keywords into output file
*/
void RMF::writeAttributes(DataSet * spectrumset, Table * outtab) {

  /* Simply copy everything from the source spectrum file top
     level header to the output table */
   outtab->addAttributes(spectrumset);

  // Create list of keywords to copy
  /*
   vector<string> keylist;
   keylist.push_back("TELESCOP");
   keylist.push_back("INSTRUME");
   keylist.push_back("SLCTEXPR");
   keylist.push_back("FILTER");
   keylist.push_back("EXPOSURE");
   keylist.push_back("OBS_ID");
   keylist.push_back("EXP_ID");
   keylist.push_back("SUBMODE");
   keylist.push_back("DATE-OBS");
   keylist.push_back("DATE-END");
   keylist.push_back("OBSERVER");
   keylist.push_back("RA_OBJ");
   keylist.push_back("DEC_OBJ");

   for (unsigned i=0; i<keylist.size(); ++i) {

      // Check if keyword exists in input table
       if (spectrumset->hasAttribute(keylist[i])) {

           *outtab->addAttribute(
                           spectrumset->attribute(keylist[i]) );
       }
   }
*/
}

Response* RMF::getRSP()
{
	return _rsp;
}


string RMF::getFileName()
{
	return _fileName;
}





