// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*
 * RMF.h
 *
 *  Created on: Mar 11, 2013
 *      Author: aibarra
 */

#ifndef RMF_H
#define RMF_H

#include <iostream>
#include <vector>
#include "Dal.h"

using namespace std;

struct Response {
	vector<vector<double> > matrix;
	vector<double> elo,
		       ehi,
		       eblo,
		       ebhi;

};

class RMF {

public:
	RMF(){_fileName = "";}
	~RMF(){delete _rsp;}
	RMF(std::string &fileName);

	/* Methods */
	Response* fixrmf( string &rmfname ) const;

	/* Purpose:
			To read in the RMF and set the matrix
	       Input:
			rmfname - name of the input response matrix file
	 */

	void writeAttributes(DataSet * spectrumset, Table * outtab);

	/* Purpose:
	                To copy keywords from input header to output file
	       Input:
	                spectrumset - reference to primary header of spectrum
	                outtab - name of the output table
	 */

	Response* getRSP();

	string getFileName();


private:
	Response *_rsp;
	string _fileName;
};

#endif /* RMF_H */


